do
	if not luup.ra_server then
		luup.task ("startup", 2, "findvera.com service is not configured", 2 )
		return false
	end

	local em_serv = 'urn:micasaverde-com:serviceId:EnergyMetering1'
	local sp_serv = 'urn:upnp-org:serviceId:SwitchPower1'
	local dim_serv= 'urn:upnp-org:serviceId:Dimming1'
	local srv_url = ('https://%s/report_energy_log_1.php'):format (luup.ra_server)
	local room_srv_url = ('https://%s/report_energy_device_room_1.php'):format (luup.ra_server)

	local userSuppliedWattage = {}
-- 		local lastPowerChange = {}
	local digestedWattage = {}
	local runningTotal = 0

	local function num(n)
		local t = type(n)
		if t=='number' then return n end
		if t=='string' then return tonumber(n) end
		return 0
	end

	function w_watts (lul_device, lul_service, lul_variable, lul_value_old, lul_value_new)
		luup.log (("w_watts(%s/%s/%s/%s/%s)"):format (lul_device, lul_service, lul_variable, tostring(lul_value_old), tostring(lul_value_new)))
		local now = os.time()
		local dw = digestedWattage[lul_device]
		local tspan = os.difftime (now, dw.lastPowerChange)
		local usedEnergy = num(lul_value_old) * tspan
		runningTotal = runningTotal + usedEnergy

--		luup.variable_set (my_serv, 'runningTotal', tostring(runningTotal), mydevnum)
		dw.curWatt = num(lul_value_new)
		dw.lastPowerChange = now
		dw.total = dw.total + usedEnergy
		if num(lul_value_new) > dw.peak then
			dw.peak = num(lul_value_new)
		end
		luup.log (("watts(%s):[%s,%s]"):format(lul_device, tostring(dw.total), tostring(dw.peak)))
	end

	function w_switch (lul_device, lul_service, lul_variable, lul_value_old, lul_value_new)
		luup.log (("w_switch(%s/%s/%s/%s/%s)"):format (tostring(lul_device), lul_service, lul_variable, tostring(lul_value_old), tostring(lul_value_new)))
		local prevwatts = userSuppliedWattage[lul_device] * num(lul_value_old)
		local newwatts = userSuppliedWattage[lul_device] * num(lul_value_new)
		local level = luup.variable_get (dim_serv, 'LoadLevelStatus', lul_device)
		if level ~= nil then
			newwatts = userSuppliedWattage[lul_device] * num(lul_value_new) * num(level) / 100.0
		end
		luup.variable_set (em_serv, "Watts", tostring (newwatts), lul_device)
		return w_watts (lul_device, lul_service, lul_variable, prevwatts, newwatts)
	end

	function w_dimlevl (lul_device, lul_service, lul_variable, lul_value_old, lul_value_new)
		luup.log (("w_dimlevl(%s/%s/%s/%s/%s)"):format (tostring(lul_device), lul_service, lul_variable, tostring(lul_value_old), tostring(lul_value_new)))
		local prevwatts = userSuppliedWattage[lul_device] * num(lul_value_old) / 100.0
		local newwatts = userSuppliedWattage[lul_device] * num(lul_value_new) / 100.0
		luup.variable_set (em_serv, "Watts", tostring (newwatts), lul_device)
		return w_watts (lul_device, lul_service, lul_variable, prevwatts, newwatts)
	end


	do
		local filename
		function doOpenELogF ()
			filename = ("/tmp/CurlQueue/LogEnergyUsageOnServer_%d.post"):format (os.time())
			do
				local flck = assert (io.open (filename..".lock", "wb"))
				flck:write (" ")
				flck:close()
			end
			return io.open (filename, "wb")
		end

		function doCloseF (f)
			f:close()
			os.remove (filename..".lock")
		end
	end

	function writeLogHeader (f, url)
		f:write ("EXPECTED_RESPONSE=OK\n")
		f:write (("CURLOPT_URL=%s\n"):format(url))
		f:write ("CURLOPT_TIMEOUT=10\n")
		f:write ("CURLOPT_SSL_VERIFYPEER=0\n")
		f:write (("PK_AccessPoint=%d\n"):format (tostring(luup.pk_accesspoint)))
		f:write (("HW_Key=%s\n"):format(luup.hw_key))
		f:write (("DateTime=%s\n"):format (os.date('%Y-%m-%d %H:%M:%S')))
	end

	function p_saveELog (lul_data)
		local f = doOpenELogF ()
		luup.log ('saving energy summary')
		writeLogHeader (f, srv_url)
		local now = os.time()
		for k,v in pairs (digestedWattage) do
			-- dummy change
			w_watts (k, em_serv, "Watts", tostring(v.curWatt), tostring(v.curWatt))
			local tspan = (os.difftime (now, v.startRep))
			if tspan > 0 then
				local avg = v.total / (os.difftime (now, v.startRep))
				f:write (("PK_Device_%d=%s,%s\n"):format (k, tostring(avg), tostring(v.peak)))
			end
			v.total = 0
			v.startRep = now
			v.peak = 0
		end
		doCloseF (f)
		luup.log ('done')
		luup.call_timer ('p_saveELog', 1, '30s', '','')
		return 0
	end

	function p_saveRooms(whichDevices)
		local f = doOpenELogF ()
		luup.log ('saving room/device summary')
		writeLogHeader (f, room_srv_url)
		for _,devnum in ipairs (whichDevices) do
			local dv = luup.devices[devnum]
			f:write (("PK_Device_%d=%d,%d,%d,%s\n"):format (devnum,
				tonumber(dv.room_num), tonumber(0),tonumber(dv.category_num), dv.description))
		end
		doCloseF (f)
		luup.log ('done saving romm/device summary')
	end

	function start_monit (lul_device)
		local now = os.time()
		luup.log (string.format('Initalizing EnergyMonitor', lul_device))
		local watchedDevices = {}
		for k,v in pairs (luup.devices) do
			if luup.device_supports_service (em_serv, k) then
				digestedWattage[k] = {avg=0, peak=0, total=0, startRep=now, lastPowerChange=now, curWatt=0}
				luup.log (string.format('monitoring device #%s:%s', k, v.description))
				if num(luup.variable_get (em_serv, 'ActualUsage', k)) > 0 then
					luup.log ('watching "watts"')
					luup.variable_watch ('w_watts', em_serv, 'Watts', k)
					digestedWattage[k].curWatt = num (luup.variable_get (em_serv, 'Watts', k))
					table.insert (watchedDevices, k)
				else
					local usw = num(luup.variable_get (em_serv,'UserSuppliedWattage',k))
					if usw then
						luup.log (("userwatts:%s"):format(usw))
						userSuppliedWattage[k] = num(usw)
						if luup.device_supports_service (sp_serv, k) then
							luup.log ('watching "Status"')
							luup.variable_watch ('w_switch', sp_serv, 'Status', k)
							local curValue = luup.variable_get (sp_serv, 'Status', k)
							w_switch (k, sp_serv, 'Status', curValue, curValue)
							digestedWattage[k].curWatt = userSuppliedWattage[k] * num (curValue)
							table.insert (watchedDevices, k)
						end
						if luup.device_supports_service (dim_serv,k) then
							luup.log ('watching "LoadLevel"')
							luup.variable_watch ('w_dimlevl', dim_serv, 'LoadLevelStatus', k)
							local curValue = luup.variable_get (dim_serv, 'LoadLevelStatus', k)
							w_dimlevl (k, dim_serv, 'LoadLevelStatus', curValue, curValue)
							digestedWattage[k].curWatt = userSuppliedWattage[k] * num (curValue) / 100.0
							table.insert (watchedDevices, k)
						end
					end
				end
			end
		end
-- 			p_saveRooms (watchedDevices)
		luup.call_timer ('p_saveELog', 1, '3s', '','')
		luup.log ('EnergyMonitor initialized')
	end

	start_monit()
end